@@ -2,7 +2,7 @@
 #' @description Estimate and validate a CUB model for given ordinal responses, with covariates for explaining both the
 #'  feeling and the uncertainty components by means of logistic transform.
 #' @aliases cubpq
-#' @usage cubpq(m, ordinal, Y, W, maxiter, toler, summary)
+#' @usage cubpq(m, ordinal, Y, W, maxiter, toler)
 #' @param m Number of ordinal categories
 #' @param ordinal Vector of ordinal responses
 #' @param Y Matrix of selected covariates for explaining the uncertainty component
@@ -20,17 +20,29 @@
 #' \emph{Quality Technology and Quantitative Management}, \bold{17}(2)  149--168
 #' @keywords internal 
 
-cubpq<-function(m,ordinal,Y,W,maxiter,toler,summary){
+########################################
+cubpq<-function(m,ordinal,Y,W,maxiter,toler){
+  
   tt0<-proc.time()
   n<-length(ordinal)
+  Y<-as.matrix(Y)
+  W<-as.matrix(W)
   p<-NCOL(Y)
   q<-NCOL(W)
   aver<-mean(ordinal)
+  
+  if (ncol(Y)==1){
+    Y<-as.numeric(Y)
+  }
+  if (ncol(W)==1){
+    W<-as.numeric(W)
+  }
+  
   YY<-cbind(1,Y);     WW<-cbind(1,W);          
   #################################################################################
   freq<-tabulate(ordinal,nbins=m)
   inipaicsi<-inibest(m,freq); pai<-inipaicsi[1]; bet0<-log(pai/(1-pai)); betjj<-c(bet0,rep(0.1,p));
-  gamajj<-inibestgama(m,ordinal,W)  ### Attention !!!
+  gamajj<-inibestgama(m,factor(ordinal,ordered=TRUE),W)  
   #################################################################################
   loglikjj<-loglikcubpq(m,ordinal,Y,W,betjj,gamajj)
   # ********************************************************************
@@ -39,7 +51,7 @@ cubpq<-function(m,ordinal,Y,W,maxiter,toler,summary){
   nniter<-1
   while(nniter<=maxiter){
     loglikold<-loglikjj
-    vettn<-bitgama(m,ordinal,W,gamajj)   
+    vettn<-as.numeric(bitgama(m,factor(ordinal,ordered=TRUE),W,gamajj)   )
     aai<- -1+1/(logis(Y,betjj))   
     ttau<-1/(1+aai/(m*vettn))        
     ####################  maximize w.r.t. bet and gama    ############
@@ -67,7 +79,7 @@ cubpq<-function(m,ordinal,Y,W,maxiter,toler,summary){
   varmat<-varcovcubpq(m,ordinal,Y,W,bet,gama)
   #if(det(varmat)<=0) stop("Variance-covariance matrix NOT positive definite")
   nomi<-c(paste("beta",0:(length(bet)-1),sep="_"),paste("gamma",0:(length(gama)-1),sep="_"))
-  stime<-round(c(bet,gama),5)
+  stime<-c(bet,gama)
   nparam<-length(stime)
   if (isTRUE(varmat==matrix(NA,nrow=nparam,ncol=nparam))==TRUE){
     ddd<-cormat<-matrix(NA,nrow=nparam,ncol=nparam)
@@ -78,45 +90,16 @@ cubpq<-function(m,ordinal,Y,W,maxiter,toler,summary){
     cormat<-(ddd%*%varmat)%*%ddd  
     trvarmat<-sum(diag(varmat))
     ICOMP<- -2*loglik + nparam*log(trvarmat/nparam) - log(det(varmat))
-    errstd<-round(sqrt(diag(varmat)),5);  wald<-round(stime/errstd,5);
-    pval<-round(2*(1-pnorm(abs(wald))),20)
+    errstd<-sqrt(diag(varmat));  wald<-stime/errstd;
+    pval<-2*(1-pnorm(abs(wald)))
   }
   rownames(cormat)<-nomi;colnames(cormat)<-nomi; 
   durata<-proc.time()-tt0;durata<-durata[1];
   ####################################################################
-  # Print CUB(p,q) results of ML estimation  
-  ####################################################################
-  if (summary==TRUE){
-    cat("\n")
-    cat("==========================================================================","\n")
-    cat("=======>>> C U B (p,q) model <<<=========   ML-estimates via E-M algorithm   ","\n")
-    cat("==========================================================================","\n")
-    cat(" Covariates for pai ==> p=",p,"     and     Covariates for csi ==> q=", q,"\n")
-    cat("==========================================================================","\n")
-    cat("*** m=", m,"  *** Sample size: n=", n,"   *** Iterations=",nniter,"Maxiter=",maxiter,"\n")
-    cat("==========================================================================","\n")
-    cat("parameters  ML-estimates  stand.errors    Wald-test      p-value ","\n")
-    cat("==========================================================================","\n")
-    for(i in 1:length(nomi)){
-      cat(nomi[i],"     ",stime[i],"      ",errstd[i],"       ",wald[i],"      ",pval[i],"\n")
-    }
-    ####################################################################
-    cat("==========================================================================","\n")
-    cat("                         Parameters correlation matrix","\n") 
-    print(round(cormat,5))
-    ##############################################################################
-    cat("==========================================================================","\n")
-    cat("Log-lik(beta^,gamma^)=",round(loglik,digits=8),"\n")
-    cat("Mean Log-likelihood  =",round(loglik/n,digits=8),"\n")
-    cat("--------------------------------------------------------------------------","\n")
-    cat("AIC-CUBpq           =",round(AICCUBpq,digits=8),"\n")
-    cat("BIC-CUBpq           =",round(BICCUBpq,digits=8),"\n")
-    cat("ICOMP-CUBpq         =",round(ICOMP,digits=8),"\n")
-    cat("==========================================================================","\n")
-    cat("Elapsed time              =",durata,"seconds","=====>>>",date(),"\n")
-    cat("==========================================================================","\n")  
-    
-  }
-
-  results<-list('estimates'=stime,'loglik'=loglik,'niter'=nniter,'varmat'=varmat, 'BIC'=round(BICCUBpq,digits=8))
+  
+  results<-list('estimates'=stime,'ordinal'=ordinal,'time'=durata,
+                'loglik'=loglik,'niter'=nniter,'varmat'=varmat,
+                'BIC'=BICCUBpq)
+  #class(results)<-"cub"
+  return(results)
 }
\ No newline at end of file
